#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_form.help module

This module provides a 'form_help' content provider, which can be used to insert help text
inside any form using an :py:class:`IFormHelp <pyams_form.interfaces.form.IFormHlep>` adapter.
"""

from zope.interface import Interface, implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_form.interfaces.form import IFormHelp
from pyams_skin.layer import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces.text import IHTMLRenderer
from pyams_utils.text import text_to_html
from pyams_viewlet.viewlet import ViewContentProvider, contentprovider_config


__docformat__ = 'restructuredtext'


@contentprovider_config(name='form_help', view=Interface, layer=IPyAMSLayer)
@template_config(template='templates/help.pt', layer=IPyAMSLayer)
class HelpContentProvider(ViewContentProvider):
    """Form help content provider"""

    help = None

    def update(self):
        """Update content provider state"""
        registry = self.request.registry
        # pylint: disable=redefined-builtin
        help = self.help = registry.queryMultiAdapter((self.context, self.request, self.view),
                                                      IFormHelp)
        if help is not None:
            if help.permission and not self.request.has_permission(help.permission,
                                                                   context=self.context):
                self.help = None
            elif help.mode and (self.view.mode != help.mode):
                self.help = None


@implementer(IFormHelp)
class FormHelp(ContextRequestViewAdapter):
    """Default form help base implementation"""

    permission = FieldProperty(IFormHelp['permission'])
    mode = FieldProperty(IFormHelp['mode'])

    outer_margin = FieldProperty(IFormHelp['outer_margin'])
    status = FieldProperty(IFormHelp['status'])
    header = FieldProperty(IFormHelp['header'])
    message = FieldProperty(IFormHelp['message'])
    message_format = FieldProperty(IFormHelp['message_format'])


@adapter_config(context=(IFormHelp, IPyAMSLayer, Interface), provides=IHTMLRenderer)
class HelpRenderer(ContextRequestViewAdapter):
    """Default form help renderer"""

    def render(self, **kwargs):  # pylint: disable=unused-argument
        """Render help text to HTML"""
        message = self.request.localizer.translate(self.context.message)
        return text_to_html(message, self.context.message_format)
